@@ -21,7 +21,7 @@ def assoc_store():
 def mol_store():
     return MemoryStore()
 
-@pytest.mark.xfail(reason="Waiting on molecule updates")
+
 def test_molecules_builder(tasks_store, assoc_store, mol_store):
     stage_one = MoleculesAssociationBuilder(tasks=tasks_store, assoc=assoc_store)
     stage_one.run()
@@ -30,9 +30,8 @@ def test_molecules_builder(tasks_store, assoc_store, mol_store):
 
     assoc_store.key = "molecule_id"
 
-    stage_two = MoleculesBuilder(assoc=assoc_store, molecules=mol_store, prefix="libe")
+    stage_two = MoleculesBuilder(assoc=assoc_store, molecules=mol_store, prefix="mpcule")
     stage_two.run()
 
     assert mol_store.count() == 48
     assert mol_store.count({"deprecated": True}) == 0
-    assert mol_store.count({"molecule_id": {"$regex": "libe-"}}) == 48
